;;; $DOOMDIR/config.el -*- lexical-binding: t; -*-

;; Place your private configuration here! Remember, you do not need to run 'doom
;; sync' after modifying this file!

;; Some functionality uses this to identify you, e.g. GPG configuration, email
;; clients, file temp+bindingslates and snippets. It is optional.
(setq user-full-name "Matt Nish-Lapidus"
      user-mail-address "matt@emenel.ca")

;; See 'C-h v doom-font' for documentation and more examples of what they
;; accept. For example:
(setq doom-font (font-spec :family "JetBrains Mono" :size 14))
(setq fancy-splash-image (concat doom-user-dir "splash.png"))

;; There are two ways to load a theme. Both assume the theme is installed and
;; available. You can either set `doom-theme' or manually load a theme with the
;; `load-theme' function. This is the default:
(setq doom-theme 'doom-monokai-pro)

;; This determines the style of line numbers in effect. If set to `nil', line
;; numbers are disabled. For relative line numbers, set this to `relative'.
(setq display-line-numbers-type t)
(setq scroll-preserve-screen-position t)

(setq org-directory "~/Library/Mobile Documents/iCloud~md~obsidian/Documents/Primary/org/")

;; Modeline
;; - add current workspace name
;; - add major mode icon
(after! doom-modeline
  (setq doom-modeline-persp-name t
        doom-modeline-major-mode-icon t))

;; set default frame size on launch
(setq default-frame-alist '( (left . 0) (top . 0) (height . 48) (width . 160) ))

;; fixing compat issue for corfu in emacs28
;; (defalias 'compat-string-width 'string-width)

;; some global settings
(setq tab-always-indent t) 		; don't autocomplete with tab
(+global-word-wrap-mode +1) 		; default to soft word wrap
(delete-selection-mode 1) 		; delete stuff!
(setq delete-active-region t) 		; delete selected stuff!

;; remove org agenda from the splash screen
(assoc-delete-all "Open org-agenda" +doom-dashboard-menu-sections)

;; reduce which-key waiting
(after! which-key
  (setq which-key-idle-delay 0.3))

;; some defaults for cursor size, scrolling, and buffer handling
(setq x-stretch-cursor t
      kill-buffer-delete-auto-save-files t
      scroll-conservatively 0)

;; nicer resizing
(cond
 ((string-equal system-type "darwin")
  (setq frame-resize-pixelwise  t
        window-resize-pixelwise t)))

;; tabs! i like them
;; (setq-default indent-tabs-mode t)
(setq-default tab-width 2)

;; import all my customized keybindings based on doom evil mode, but without evil.
(load! "bindings.el")

(setq treemacs-follow-mode t)
(setq treemacs-project-follow-mode t)

;; smart tabs set modes
(after! smart-tabs-mode
  (smart-tabs-insinuate 'c 'javascript 'python))

;; lsp mode config to work with corfu/cape
(after! lsp-mode
  (setq lsp-completion-provider :none)
  (setq lsp-signature-render-documentation nil)
  (setq lsp-completion-show-kind t)

  (defun my/lsp-mode-setup-completion ()
    (setf (alist-get 'styles (alist-get 'lsp-capf completion-category-defaults))
          '(orderless)))
  (add-hook! lsp-completion-mode 'my/lsp-mode-setup-completion))

;; bibliography file location
(after! citar
  (setq! citar-bibliography '("/Users/emenel/Zotero/lib.bib"))
  (setq! citar-notes-paths '("/Users/emenel/Library/Mobile Documents/iCloud~md~obsidian/Documents/Primary/Sources"))
  (setq! citar-file-note-extensions '("md")))

;; svelte support
(add-to-list 'auto-mode-alist '("\\.svelte\\'" . web-mode))
(setq web-mode-engines-alist
      '(("svelte" . "\\.svelte\\'")))

;; custom keybind for adding html closing tag because i have muscle memory from sublime
(after! web-mode
  (map! :map web-mode-map
	"M-s-." 'web-mode-element-close))

(add-hook! web-mode (web-mode-use-tabs) (setq indent-tabs-mode t))

;; fix centaur tabs to show at the right times and add my muscle memory keybind
(after! centaur-tabs
  (setq centaur-tabs-set-bar 'right)
  (setq centaur-tabs-cycle-scope 'tabs)
  (map! :map centaur-tabs-mode-map
	"s-M-<right>" 'centaur-tabs-forward
	"s-M-<left>" 'centaur-tabs-backward))

;; use kitty for terminal-here
(setq terminal-here-mac-terminal-command '("kitty"))

;; set project paths and ignore buffers
(setq projectile-project-search-path '(("~/Source" . 2)))
(setq projectile-globally-ignored-buffers
      '("*scratch*"
	"*lsp-log*"))

;; ignoring buffers by their major mode
(setq projectile-globally-ignored-modes
      '("erc-mode"
	"help-mode"
	"completion-list-mode"
	"Buffer-menu-mode"
	"gnus-.*-mode"
	"occur-mode"))

;; mouse support for vertico buffers
(after! vertico
  (vertico-mouse-mode))

;; better completions for sly
(after! sly
  (setq sly-lisp-implementations
      '((sbcl ("sbcl"))
        (ciel ("sbcl" "--core" "/Users/emenel/quicklisp/local-projects/CIEL/ciel-core" "--eval" "(in-package :ciel-user)"))))
  (setq sly-default-lisp 'sbcl)
  (setq sly-complete-symbol-function 'sly-flex-completions))

;; make esc close completion from corfu
(map! :map corfu-map [escape] 'corfu-quit)

;; cl-collider cl-patterns emacs helpers TODO: test this
(after! lisp-mode
  (defun cl-patterns-helpers-load ()
    (interactive)
    (sly-eval-async '(cl:namestring (asdf:system-source-directory (asdf:find-system 'cl-patterns)))
		    (lambda (path)
		      (load (concat path "res/emacs/cl-patterns-helpers") nil nil nil t)
		      (load (concat path "res/emacs/cl-patterns-skeletons") nil nil nil t)))
    ;; (define-key sly-mode-map (kbd "C-c p") 'cl-patterns-play-or-end-context-or-select-pdef)
    ;; (define-key sly-mode-map (kbd "C-c P") 'cl-patterns-play-or-stop-context-or-select-pdef)
    ;; (define-key sly-mode-map (kbd "C-c s") 'cl-patterns-stop-all)
    ;; (define-key sly-doc-map (kbd "s") 'cl-patterns-supercollider-documentation))
    (add-hook 'sly-connected-hook 'cl-patterns-helpers-load)))

;; tidal cycles!
(setq tidal-boot-script-path "~/.cabal/share/aarch64-osx-ghc-9.4.2/tidal-1.9.3/BootTidal.hs")

(load! "obsidian-config.el")
